@@ -1,26 +1,27 @@
 """
-    flask_security.datastore
-    ~~~~~~~~~~~~~~~~~~~~~~~~
+flask_security.datastore
+~~~~~~~~~~~~~~~~~~~~~~~~
 
-    This module contains an user datastore classes.
+This module contains an user datastore classes.
 
-    :copyright: (c) 2012 by Matt Wright.
-    :copyright: (c) 2019-2024 by J. Christopher Wagner (jwag).
-    :license: MIT, see LICENSE for more details.
+:copyright: (c) 2012 by Matt Wright.
+:copyright: (c) 2019-2024 by J. Christopher Wagner (jwag).
+:license: MIT, see LICENSE for more details.
 """
 
 from __future__ import annotations
 
-from datetime import datetime
 import json
 import typing as t
 import uuid
 from copy import copy
 
+from .core import UserMixin, RoleMixin, WebAuthnMixin
 from .utils import config_value as cv
 
 if t.TYPE_CHECKING:  # pragma: no cover
     import flask_sqlalchemy
+    import flask_sqlalchemy_lite
     import mongoengine
     import sqlalchemy.orm.scoping
 
@@ -186,9 +187,9 @@ class UserDatastore:
 
     def __init__(
         self,
-        user_model: t.Type[User],
-        role_model: t.Type[Role],
-        webauthn_model: t.Type[WebAuthn] | None = None,
+        user_model: t.Type[UserMixin],
+        role_model: t.Type[RoleMixin],
+        webauthn_model: t.Type[WebAuthnMixin] | None = None,
     ):
         self.user_model = user_model
         self.role_model = role_model
@@ -202,7 +203,10 @@ class UserDatastore:
         def put(self, model):
             pass
 
-    def _prepare_role_modify_args(self, role: str | Role) -> Role | None:
+        def commit(self):
+            pass
+
+    def _prepare_role_modify_args(self, role: str | RoleMixin) -> RoleMixin | None:
         if isinstance(role, str):
             return self.find_role(role)
         return role
@@ -223,18 +227,21 @@ class UserDatastore:
 
         return kwargs
 
-    def find_user(self, **kwargs: t.Any) -> User | None:
+    def find_user(
+        self, case_insensitive: bool = False, **kwargs: t.Any
+    ) -> UserMixin | None:
         """Returns a user matching the provided parameters.
-        Besides keyword arguments used to filter the results,
-        'case_insensitive' can be passed (defaults to False)
+        A single kwarg will be poped and used to filter results. This should
+        be a unique/primary key in the User model since only a single result is
+        returned.
         """
         raise NotImplementedError
 
-    def find_role(self, role: str) -> Role | None:
+    def find_role(self, role: str) -> RoleMixin | None:
         """Returns a role matching the provided name."""
         raise NotImplementedError
 
-    def add_role_to_user(self, user: User, role: Role | str) -> bool:
+    def add_role_to_user(self, user: UserMixin, role: RoleMixin | str) -> bool:
         """Adds a role to a user.
 
         :param user: The user to manipulate.
@@ -250,7 +257,7 @@ class UserDatastore:
             return True
         return False
 
-    def remove_role_from_user(self, user: User, role: Role | str) -> bool:
+    def remove_role_from_user(self, user: UserMixin, role: RoleMixin | str) -> bool:
         """Removes a role from a user.
 
         :param user: The user to manipulate. Can be an User object or email
@@ -268,7 +275,7 @@ class UserDatastore:
         return rv
 
     def add_permissions_to_role(
-        self, role: Role | str, permissions: set | list | tuple | str
+        self, role: RoleMixin | str, permissions: set | list | tuple | str
     ) -> bool:
         """Add one or more permissions to role.
 
@@ -296,7 +303,7 @@ class UserDatastore:
         return rv
 
     def remove_permissions_from_role(
-        self, role: Role | str, permissions: set | list | tuple | str
+        self, role: RoleMixin | str, permissions: set | list | tuple | str
     ) -> bool:
         """Remove one or more permissions from a role.
 
@@ -322,13 +329,13 @@ class UserDatastore:
             self.put(role_obj)
         return rv
 
-    def toggle_active(self, user: User) -> bool:
+    def toggle_active(self, user: UserMixin) -> bool:
         """Toggles a user's active status. Always returns True."""
         user.active = not user.active
         self.put(user)
         return True
 
-    def deactivate_user(self, user: User) -> bool:
+    def deactivate_user(self, user: UserMixin) -> bool:
         """Deactivates a specified user. Returns `True` if a change was made.
 
         This will immediately disallow access to all endpoints that require
@@ -343,7 +350,7 @@ class UserDatastore:
             return True
         return False
 
-    def activate_user(self, user: User) -> bool:
+    def activate_user(self, user: UserMixin) -> bool:
         """Activates a specified user. Returns `True` if a change was made.
 
         :param user: The user to activate
@@ -354,7 +361,7 @@ class UserDatastore:
             return True
         return False
 
-    def set_uniquifier(self, user: User, uniquifier: str | None = None) -> None:
+    def set_uniquifier(self, user: UserMixin, uniquifier: str | None = None) -> None:
         """Set user's Flask-Security identity key.
         This will immediately render outstanding auth tokens,
         session cookies and remember cookies invalid.
@@ -369,7 +376,9 @@ class UserDatastore:
         user.fs_uniquifier = uniquifier
         self.put(user)
 
-    def set_token_uniquifier(self, user: User, uniquifier: str | None = None) -> None:
+    def set_token_uniquifier(
+        self, user: UserMixin, uniquifier: str | None = None
+    ) -> None:
         """Set user's auth token identity key.
         This will immediately render outstanding auth tokens invalid.
 
@@ -387,7 +396,7 @@ class UserDatastore:
             user.fs_token_uniquifier = uniquifier
             self.put(user)
 
-    def create_role(self, **kwargs: t.Any) -> Role:
+    def create_role(self, **kwargs: t.Any) -> RoleMixin:
         """
         Creates and returns a new role from the given parameters.
         Supported params (depending on RoleModel):
@@ -414,13 +423,13 @@ class UserDatastore:
         role = self.role_model(**kwargs)
         return self.put(role)
 
-    def find_or_create_role(self, name: str, **kwargs: t.Any) -> Role:
+    def find_or_create_role(self, name: str, **kwargs: t.Any) -> RoleMixin:
         """Returns a role matching the given name or creates it with any
         additionally provided parameters.
         """
         return self.find_role(name) or self.create_role(name=name, **kwargs)
 
-    def create_user(self, **kwargs: t.Any) -> User:
+    def create_user(self, **kwargs: t.Any) -> UserMixin:
         """Creates and returns a new user from the given parameters.
 
         :kwparam email: required.
@@ -437,7 +446,7 @@ class UserDatastore:
             Best practice is::
 
                 try:
-                    enorm = app.security._mail_util.validate(email)
+                    enorm = app.security.mail_util.validate(email)
                 except ValueError:
 
         .. danger::
@@ -450,7 +459,7 @@ class UserDatastore:
 
            Best practice is::
 
-            pbad, pnorm = app.security._password_util.validate(password, True)
+            pbad, pnorm = app.security.password_util.validate(password, True)
 
            Look for `pbad` being None. Pass the normalized password `pnorm` to this
            method.
@@ -462,14 +471,14 @@ class UserDatastore:
         user = self.user_model(**kwargs)
         return self.put(user)
 
-    def delete_user(self, user: User) -> None:
+    def delete_user(self, user: UserMixin) -> None:
         """Deletes the specified user.
 
         :param user: The user to delete
         """
         self.delete(user)  # type: ignore
 
-    def reset_user_access(self, user: User) -> None:
+    def reset_user_access(self, user: UserMixin) -> None:
         """
         Use this method to reset user authentication methods in the case of compromise.
         This will:
@@ -510,7 +519,7 @@ class UserDatastore:
 
     def tf_set(
         self,
-        user: User,
+        user: UserMixin,
         primary_method: str,
         totp_secret: str | None = None,
         phone: str | None = None,
@@ -524,7 +533,7 @@ class UserDatastore:
         This could be called from an application to apiori setup a user for two factor
         without the user having to go through the setup process.
 
-        To get a totp_secret - use ``app.security._totp_factory.generate_totp_secret()``
+        To get a totp_secret - use ``app.security.totp_factory.generate_totp_secret()``
 
         .. versionadded: 3.4.1
         """
@@ -542,7 +551,7 @@ class UserDatastore:
         if changed:
             self.put(user)
 
-    def tf_reset(self, user: User) -> None:
+    def tf_reset(self, user: UserMixin) -> None:
         """Disable two-factor auth for user.
 
         .. versionadded: 3.4.1
@@ -552,7 +561,7 @@ class UserDatastore:
         user.tf_phone_number = None
         self.put(user)
 
-    def mf_set_recovery_codes(self, user: User, rcs: list[str] | None) -> None:
+    def mf_set_recovery_codes(self, user: UserMixin, rcs: list[str] | None) -> None:
         """Set MF recovery codes into user record.
         Any existing codes will be erased.
 
@@ -561,11 +570,11 @@ class UserDatastore:
         user.mf_recovery_codes = rcs
         self.put(user)
 
-    def mf_get_recovery_codes(self, user: User) -> list[str]:
+    def mf_get_recovery_codes(self, user: UserMixin) -> list[str]:
         codes = getattr(user, "mf_recovery_codes", [])
         return codes if codes else []
 
-    def mf_delete_recovery_code(self, user: User, idx: int) -> bool:
+    def mf_delete_recovery_code(self, user: UserMixin, idx: int) -> bool:
         """Delete a single recovery code.
         Recovery codes are single-use - so delete after using!
 
@@ -582,7 +591,7 @@ class UserDatastore:
         except IndexError:
             return False
 
-    def us_get_totp_secrets(self, user: User) -> dict[str, str]:
+    def us_get_totp_secrets(self, user: UserMixin) -> dict[str, str]:
         """Return totp secrets.
         These are json encoded in the DB.
 
@@ -594,7 +603,9 @@ class UserDatastore:
             return {}
         return json.loads(user.us_totp_secrets)
 
-    def us_put_totp_secrets(self, user: User, secrets: dict[str, str] | None) -> None:
+    def us_put_totp_secrets(
+        self, user: UserMixin, secrets: dict[str, str] | None
+    ) -> None:
         """Save secrets. Assume to be a dict (or None)
         with keys as methods, and values as (encrypted) secrets.
 
@@ -605,7 +616,7 @@ class UserDatastore:
 
     def us_set(
         self,
-        user: User,
+        user: UserMixin,
         method: str,
         totp_secret: str | None = None,
         phone: str | None = None,
@@ -618,7 +629,7 @@ class UserDatastore:
         This could be called from an application to apiori setup a user for unified
         sign in without the user having to go through the setup process.
 
-        To get a totp_secret - use ``app.security._totp_factory.generate_totp_secret()``
+        To get a totp_secret - use ``app.security.totp_factory.generate_totp_secret()``
 
         .. versionadded:: 3.4.1
         """
@@ -631,7 +642,7 @@ class UserDatastore:
             user.us_phone_number = phone
             self.put(user)
 
-    def us_reset(self, user: User, method: str | None = None) -> None:
+    def us_reset(self, user: UserMixin, method: str | None = None) -> None:
         """Disable unified sign in for user.
         This will disable authenticator app and SMS, and email.
         N.B. if user has no password they may not be able to authenticate at all.
@@ -655,19 +666,19 @@ class UserDatastore:
                 user.us_phone_number = None
                 self.put(user)
 
-    def us_setup_email(self, user: User) -> bool:
+    def us_setup_email(self, user: UserMixin) -> bool:
         # setup email (if allowed) for user for unified sign in.
         from .proxies import _security
 
         if not cv("UNIFIED_SIGNIN") or "email" not in cv("US_ENABLED_METHODS"):
             return False
         totp_secrets = self.us_get_totp_secrets(user)
-        totp_secrets["email"] = _security._totp_factory.generate_totp_secret()
+        totp_secrets["email"] = _security.totp_factory.generate_totp_secret()
         self.us_put_totp_secrets(user, totp_secrets)
         return True
 
     def set_webauthn_user_handle(
-        self, user: User, user_handle: str | None = None
+        self, user: UserMixin, user_handle: str | None = None
     ) -> None:
         """Set the value for the Relaying Party's (that's us)
         UserHandle (user.id)
@@ -680,7 +691,7 @@ class UserDatastore:
 
     def create_webauthn(
         self,
-        user: User,
+        user: UserMixin,
         credential_id: bytes,
         public_key: bytes,
         name: str,
@@ -701,20 +712,20 @@ class UserDatastore:
         """
         raise NotImplementedError
 
-    def delete_webauthn(self, webauthn: WebAuthn) -> None:
+    def delete_webauthn(self, webauthn: WebAuthnMixin) -> None:
         """
         .. versionadded: 5.0.0
         """
         self.delete(webauthn)
 
-    def find_webauthn(self, credential_id: bytes) -> WebAuthn | None:
+    def find_webauthn(self, credential_id: bytes) -> WebAuthnMixin | None:
         """Returns a credential matching the id.
 
         .. versionadded: 5.0.0
         """
         raise NotImplementedError
 
-    def find_user_from_webauthn(self, webauthn: WebAuthn) -> User | None:
+    def find_user_from_webauthn(self, webauthn: WebAuthnMixin) -> UserMixin | None:
         """Returns user associated with this webauthn credential
 
         .. versionadded: 5.0.0
@@ -724,7 +735,7 @@ class UserDatastore:
         user_filter = webauthn.get_user_mapping()
         return self.find_user(**user_filter)
 
-    def webauthn_reset(self, user: User) -> None:
+    def webauthn_reset(self, user: UserMixin) -> None:
         """Reset access via webauthn credentials.
         This will DELETE all registered credentials.
         There doesn't appear to be any reason to change the user's
@@ -752,47 +763,55 @@ class SQLAlchemyUserDatastore(SQLAlchemyDatastore, UserDatastore):
     def __init__(
         self,
         db: flask_sqlalchemy.SQLAlchemy,
-        user_model: t.Type[User],
-        role_model: t.Type[Role],
-        webauthn_model: t.Type[WebAuthn] | None = None,
+        user_model: t.Type[UserMixin],
+        role_model: t.Type[RoleMixin],
+        webauthn_model: t.Type[WebAuthnMixin] | None = None,
     ):
         SQLAlchemyDatastore.__init__(self, db)
         UserDatastore.__init__(self, user_model, role_model, webauthn_model)
 
-    def find_user(self, case_insensitive: bool = False, **kwargs: t.Any) -> User | None:
-        from sqlalchemy import func as alchemyFn
+    def find_user(
+        self, case_insensitive: bool = False, **kwargs: t.Any
+    ) -> UserMixin | None:
+        from sqlalchemy import func, select
+        from sqlalchemy.orm import joinedload
+
+        attr, value = kwargs.popitem()  # only a single query attribute accepted
+        val = getattr(self.user_model, attr)
+        stmt = select(self.user_model)
 
-        query = self.user_model.query
         if cv("JOIN_USER_ROLES") and hasattr(self.user_model, "roles"):
-            from sqlalchemy.orm import joinedload
-
-            query = query.options(joinedload(self.user_model.roles))  # type: ignore
-
+            stmt = stmt.options(joinedload(self.user_model.roles))  # type: ignore
         if case_insensitive:
-            # While it is of course possible to pass in multiple keys to filter on
-            # that isn't the normal use case. If caller asks for case_insensitive
-            # AND gives multiple keys - throw an error.
-            if len(kwargs) > 1:
-                raise ValueError("Case insensitive option only supports single key")
-            attr, identifier = kwargs.popitem()
-            subquery = alchemyFn.lower(
-                getattr(self.user_model, attr)
-            ) == alchemyFn.lower(identifier)
-            return query.filter(subquery).first()
+            stmt = stmt.where(
+                func.lower(val) == func.lower(value)  # type: ignore[arg-type]
+            )
         else:
-            return query.filter_by(**kwargs).first()
+            stmt = stmt.where(val == value)  # type: ignore[arg-type]
+        return self.db.session.scalar(stmt)
 
-    def find_role(self, role: str) -> Role | None:
-        return self.role_model.query.filter_by(name=role).first()  # type: ignore
+    def find_role(self, role: str) -> RoleMixin | None:
+        from sqlalchemy import select
 
-    def find_webauthn(self, credential_id: bytes) -> WebAuthn | None:
-        return self.webauthn_model.query.filter_by(  # type: ignore
-            credential_id=credential_id
-        ).first()
+        return self.db.session.scalar(
+            select(self.role_model).where(self.role_model.name == role)  # type: ignore
+        )
+
+    def find_webauthn(self, credential_id: bytes) -> WebAuthnMixin | None:
+        from sqlalchemy import select
+
+        if not self.webauthn_model:  # pragma: no cover
+            raise NotImplementedError
+
+        return self.db.session.scalar(
+            select(self.webauthn_model).where(
+                self.webauthn_model.credential_id == credential_id  # type: ignore
+            )
+        )
 
     def create_webauthn(
         self,
-        user: User,
+        user: UserMixin,
         credential_id: bytes,
         public_key: bytes,
         name: str,
@@ -841,9 +860,9 @@ class SQLAlchemySessionUserDatastore(SQLAlchemyUserDatastore, SQLAlchemyDatastor
     def __init__(
         self,
         session: sqlalchemy.orm.scoping.scoped_session,
-        user_model: t.Type[User],
-        role_model: t.Type[Role],
-        webauthn_model: t.Type[WebAuthn] | None = None,
+        user_model: t.Type[UserMixin],
+        role_model: t.Type[RoleMixin],
+        webauthn_model: t.Type[WebAuthnMixin] | None = None,
     ):
         class PretendFlaskSQLAlchemyDb:
             """This is a pretend db object, so we can just pass in a session."""
@@ -859,8 +878,35 @@ class SQLAlchemySessionUserDatastore(SQLAlchemyUserDatastore, SQLAlchemyDatastor
             webauthn_model,
         )
 
-    def commit(self):
-        super().commit()
+
+class FSQLALiteUserDatastore(SQLAlchemyUserDatastore, UserDatastore):
+    """A UserDatastore implementation that assumes the use of
+    `Flask-SQLAlchemy-Lite <https://pypi.python.org/pypi/flask-sqlalchemy-lite/>`_
+    for datastore transactions.
+
+    :param db:
+    :param user_model: See :ref:`Models <models_topic>`.
+    :param role_model: See :ref:`Models <models_topic>`.
+    :param webauthn_model: See :ref:`Models <models_topic>`.
+    """
+
+    if t.TYPE_CHECKING:  # pragma: no cover
+        from flask_sqlalchemy_lite import SQLAlchemy
+
+    def __init__(
+        self,
+        db: flask_sqlalchemy_lite.SQLAlchemy,
+        user_model: t.Type[UserMixin],
+        role_model: t.Type[RoleMixin],
+        webauthn_model: t.Type[WebAuthnMixin] | None = None,
+    ):
+        SQLAlchemyUserDatastore.__init__(
+            self,
+            db,  # type: ignore
+            user_model,
+            role_model,
+            webauthn_model,
+        )
 
 
 class MongoEngineUserDatastore(MongoEngineDatastore, UserDatastore):
@@ -878,9 +924,9 @@ class MongoEngineUserDatastore(MongoEngineDatastore, UserDatastore):
     def __init__(
         self,
         db: mongoengine.connection,
-        user_model: t.Type[User],
-        role_model: t.Type[Role],
-        webauthn_model: t.Type[WebAuthn] | None = None,
+        user_model: t.Type[UserMixin],
+        role_model: t.Type[RoleMixin],
+        webauthn_model: t.Type[WebAuthnMixin] | None = None,
     ):
         MongoEngineDatastore.__init__(self, db)
         UserDatastore.__init__(self, user_model, role_model, webauthn_model)
@@ -897,6 +943,9 @@ class MongoEngineUserDatastore(MongoEngineDatastore, UserDatastore):
                 if len(kwargs) > 1:
                     raise ValueError("Case insensitive option only supports single key")
                 attr, identifier = kwargs.popitem()
+                # Mongo doesn't like __iexact compares with None
+                if identifier is None:
+                    return None
                 query = {f"{attr}__iexact": identifier}
                 obj = self.user_model.objects(**query).first()
             else:
@@ -910,7 +959,7 @@ class MongoEngineUserDatastore(MongoEngineDatastore, UserDatastore):
     def find_role(self, role):
         return self.role_model.objects(name=role).first()
 
-    def find_webauthn(self, credential_id: bytes) -> WebAuthn | None:
+    def find_webauthn(self, credential_id: bytes) -> WebAuthnMixin | None:
         if not self.webauthn_model:
             raise NotImplementedError
 
@@ -921,7 +970,7 @@ class MongoEngineUserDatastore(MongoEngineDatastore, UserDatastore):
 
     def create_webauthn(
         self,
-        user: User,
+        user: UserMixin,
         credential_id: bytes,
         public_key: bytes,
         name: str,
@@ -1058,7 +1107,7 @@ class PeeweeUserDatastore(PeeweeDatastore, UserDatastore):
 
     def create_webauthn(
         self,
-        user: User,
+        user: UserMixin,
         credential_id: bytes,
         public_key: bytes,
         name: str,
@@ -1137,66 +1186,3 @@ class PonyUserDatastore(PonyDatastore, UserDatastore):
     @with_pony_session
     def create_role(self, **kwargs):
         return super().create_role(**kwargs)
-
-
-if t.TYPE_CHECKING:  # pragma: no cover
-    # Normally - the application creates the Models and glues them together
-    # For typing we do that here since we don't know which DB interface they
-    # will pick.
-    from .core import UserMixin, RoleMixin, WebAuthnMixin
-
-    class CanonicalUserDatastore(Datastore, UserDatastore):
-        pass
-
-    class User(UserMixin):
-        id: int
-        email: str
-        username: str | None
-        password: str | None
-        active: bool
-        fs_uniquifier: str
-        fs_token_uniquifier: str
-        fs_webauthn_user_handle: str
-        confirmed_at: datetime | None
-        last_login_at: datetime
-        current_login_at: datetime
-        last_login_ip: str | None
-        current_login_ip: str | None
-        login_count: int
-        tf_primary_method: str | None
-        tf_totp_secret: str | None
-        tf_phone_number: str | None
-        mf_recovery_codes: list[str] | None
-        us_phone_number: str | None
-        us_totp_secrets: str | bytes | None
-        create_datetime: datetime
-        update_datetime: datetime
-        roles: list[Role]
-        webauthn: list[WebAuthn]
-
-        def __init__(self, **kwargs): ...
-
-    class Role(RoleMixin):
-        id: int
-        name: str
-        description: str | None
-        permissions: list[str] | None
-        update_datetime: datetime
-
-        def __init__(self, **kwargs): ...
-
-    class WebAuthn(WebAuthnMixin):
-        id: int
-        name: str
-        credential_id: bytes
-        public_key: bytes
-        sign_count: int
-        transports: list[str] | None
-        backup_state: bool
-        device_type: str
-        extensions: str | None
-        lastuse_datetime: datetime
-        user_id: int
-        usage: str
-
-        def __init__(self, **kwargs): ...
