@@ -1,12 +1,12 @@
 """
-    flask_security.decorators
-    ~~~~~~~~~~~~~~~~~~~~~~~~~
+flask_security.decorators
+~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    Flask-Security decorators module
+Flask-Security decorators module
 
-    :copyright: (c) 2012-2019 by Matt Wright.
-    :copyright: (c) 2019-2024 by J. Christopher Wagner (jwag).
-    :license: MIT, see LICENSE for more details.
+:copyright: (c) 2012-2019 by Matt Wright.
+:copyright: (c) 2019-2024 by J. Christopher Wagner (jwag).
+:license: MIT, see LICENSE for more details.
 """
 
 from __future__ import annotations
@@ -48,22 +48,6 @@ _csrf = LocalProxy(lambda: current_app.extensions["csrf"])
 
 BasicAuth = namedtuple("BasicAuth", "username, password")
 
-# NOTE: this is here for backwards compatibility, it is deprecated and
-# could be removed any time!
-_default_unauthenticated_html = """
-    <h1>Unauthorized</h1>
-    <p>The server could not verify that you are authorized to access the URL
-    requested. You either supplied the wrong credentials (e.g. a bad password),
-    or your browser doesn't understand how to supply the credentials required.
-    </p>
-    """
-
-
-def _get_unauthenticated_response(text=None, headers=None):
-    text = text or _default_unauthenticated_html
-    headers = headers or {}
-    return Response(text, 401, headers)
-
 
 def default_unauthn_handler(mechanisms=None, headers=None):
     """Default callback for failures to authenticate
@@ -79,8 +63,6 @@ def default_unauthn_handler(mechanisms=None, headers=None):
     headers = headers or {}
     m, c = get_message("UNAUTHENTICATED")
 
-    if cv("BACKWARDS_COMPAT_UNAUTHN"):
-        return _get_unauthenticated_response(headers=headers)
     if _security._want_json(request):
         payload = json_error_response(errors=m)
         return _security._render_json(payload, 401, headers, None)
@@ -221,14 +203,18 @@ def handle_csrf(method: str, json_response: bool = False) -> ResponseValue | Non
     if the request is JSON - it MUST send the csrf_token as a header.
 
     If the passed in method is not in
-    :py:data:`SECURITY_CSRF_PROTECT_MECHANISMS` then not only
-    will no CSRF code be run, but a flag in the current context ``fs_ignore_csrf``
-    will be set so that downstream code knows to ignore any CSRF checks.
+    :py:data:`SECURITY_CSRF_PROTECT_MECHANISMS` then in addition to
+    no CSRF code being run, the flask_wtf request global 'csrf_valid' will be set
+    so that downstream code knows to ignore any CSRF checks.
 
     Returns None if all ok, returns a Response with JSON error if request
     wanted JSON - else re-raises the CSRFError exception.
 
     .. versionadded:: 3.3.0
+
+    .. versionchanged:: 5.4.3
+        Use flask_wtf request global 'csrf_valid' instead of our own to handle
+        application forms that aren't derived from our forms.
     """
     if (
         not current_app.config.get("WTF_CSRF_ENABLED", False)
@@ -247,7 +233,7 @@ def handle_csrf(method: str, json_response: bool = False) -> ResponseValue | Non
                     return _security._render_json(payload, 400, None, None)
                 raise
             return None
-    set_request_attr("fs_ignore_csrf", True)
+    set_request_attr("csrf_valid", True)  # flask_wtf global
     return None
 
 
@@ -328,9 +314,7 @@ def auth_required(
         timedelta.total_seconds() is used for the calculations:
 
             - If > 0, then the caller must have authenticated within the time specified
-              (as measured using the session cookie).
-            - If 0 and not within the grace period (see below) the caller will
-              always be redirected to re-authenticate.
+              (as measured using the session cookie or authentication token).
             - If < 0 (the default) no freshness check is performed.
 
         Note that Basic Auth, by definition, is always 'fresh' and will never result in
@@ -418,8 +402,7 @@ def auth_required(
             for method, mechanism in mechanisms:
                 if mechanism and mechanism():
                     # successfully authenticated. Basic auth is by definition 'fresh'.
-                    # Note that using token auth is ok - but caller still has to pass
-                    # in a session cookie...
+                    # If 'within' is set - check for freshness of authentication.
                     if not check_and_update_authn_fresh(within, grace, method):
                         return _security._reauthn_handler(within, grace)
                     if eresponse := handle_csrf(method, _security._want_json(request)):
@@ -447,39 +430,42 @@ def unauth_csrf(
 
     This decorator does nothing if *WTF_CSRF_ENABLED* == **False**.
 
-    This decorator will always require CSRF if the caller is authenticated.
-
     This decorator will suppress CSRF if caller isn't authenticated and has set the
-    :py:data:`SECURITY_CSRF_IGNORE_UNAUTH_ENDPOINTS` config variable.
+    :py:data:`SECURITY_CSRF_IGNORE_UNAUTH_ENDPOINTS` config variable to **True**.
 
-    :param fall_through: if set to True, then if CSRF fails here - simply keep going.
-        This is appropriate if underlying view is form based and once the form is
-        instantiated, the csrf_token will be available.
-        Note that this can mask some errors such as 'The CSRF session token is missing.'
-        meaning that the caller didn't send a session cookie and instead the caller
-        might get a 'The CSRF token is missing.' error.
+    If the caller IS authenticated, it will perform normal CSRF checks. This makes
+    this decorator useful for the views that can be called EITHER when authenticated
+    or anonymously.
 
     .. versionadded:: 3.3.0
+
+    .. versionchanged:: 5.4.3
+        The fall_through parameter is now ignored.
+        Add code to properly handle JSON errors.
     """
 
     def wrapper(fn):
         @wraps(fn)
         def decorated(*args, **kwargs):
-            if not current_app.config.get(
-                "WTF_CSRF_ENABLED", False
-            ) or not current_app.extensions.get("csrf", None):
+            if (
+                not current_app.config.get("WTF_CSRF_ENABLED", False)
+                or not current_app.extensions.get("csrf", None)
+                or g.get("csrf_valid", False)
+            ):
                 return current_app.ensure_sync(fn)(*args, **kwargs)
 
             if cv("CSRF_IGNORE_UNAUTH_ENDPOINTS") and not is_user_authenticated(
                 current_user
             ):
-                set_request_attr("fs_ignore_csrf", True)
+                set_request_attr("csrf_valid", True)
             else:
                 try:
                     _csrf.protect()
-                except CSRFError:
-                    if not fall_through:
-                        raise
+                except CSRFError as e:
+                    if _security._want_json(request):
+                        payload = json_error_response(errors=e.description)
+                        return _security._render_json(payload, 400, None, None)
+                    raise
 
             return current_app.ensure_sync(fn)(*args, **kwargs)
 
