@@ -1,14 +1,14 @@
 """
-    flask_security.mail_util
-    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+flask_security.mail_util
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    Utility class providing methods for validating, normalizing and sending emails.
+Utility class providing methods for validating, normalizing and sending emails.
 
-    :copyright: (c) 2020-2024 by J. Christopher Wagner (jwag).
-    :license: MIT, see LICENSE for more details.
+:copyright: (c) 2020-2025 by J. Christopher Wagner (jwag).
+:license: MIT, see LICENSE for more details.
 
-    While this default implementation uses Flask-Mailman - we want to make sure that
-    Flask-Mailman isn't REQUIRED (if this implementation isn't used).
+While this default implementation uses Flask-Mail - we want to make sure that
+Flask-Mailman isn't REQUIRED (if this implementation isn't used).
 """
 
 from __future__ import annotations
@@ -18,12 +18,23 @@ import typing as t
 import email_validator
 from flask import current_app
 
-from .utils import config_value
+from .utils import config_value, get_message
 
 if t.TYPE_CHECKING:  # pragma: no cover
     import flask
 
 
+class EmailValidateException(ValueError):
+    """This is raised for any email validation errors.
+    This can be used by custom MailUtil implementations to provide
+    custom error messages.
+    """
+
+    def __init__(self, message: str) -> None:
+        super().__init__(message)
+        self.msg = message
+
+
 class MailUtil:
     """
     Utility class providing methods for validating, normalizing and sending emails.
@@ -69,7 +80,7 @@ class MailUtil:
         so we cast to str() here to force localization.
         """
 
-        if current_app.extensions.get("mailman", None):
+        if current_app.extensions.get("mailman", None):  # pragma: no cover
             from flask_mailman import EmailMultiAlternatives, Mail
 
             # Flask-Mailman doesn't appear to take a tuple - a bug has been filed
@@ -94,7 +105,7 @@ class MailUtil:
                     msg.attach_alternative(html, "text/html")
                 msg.send()
 
-        elif current_app.extensions.get("mail", None):  # pragma: no cover
+        elif current_app.extensions.get("mail", None):
             from flask_mail import Message
 
             # In Flask-Mail, sender can be a two element tuple -- (name, address)
@@ -114,24 +125,24 @@ class MailUtil:
 
     def normalize(self, email: str) -> str:
         """
-        Given an input email - return a normalized version or raises ValueError
-        if field value isn't syntactically valid.
+        Given an input email - return a normalized version or
+        raise EmailValidateException if field value isn't syntactically valid.
 
-        This is called for forms that use email as an identity to be looked up.
+        This is called by forms that use email as an identity to be looked up.
 
         Must be called in app context and uses :py:data:`SECURITY_EMAIL_VALIDATOR_ARGS`
         config variable to pass any relevant arguments to
         email_validator.validate_email() method.
 
         This defaults to NOT checking for deliverability (i.e. DNS checks).
-
-        Will throw email_validator.EmailNotValidError (ValueError)
-        if email isn't syntactically valid.
         """
         validator_args = config_value("EMAIL_VALIDATOR_ARGS") or {}
         validator_args["check_deliverability"] = False
-        valid = email_validator.validate_email(email, **validator_args)
-        return valid.normalized
+        try:
+            valid = email_validator.validate_email(email, **validator_args)
+            return valid.normalized
+        except ValueError:
+            raise EmailValidateException(get_message("INVALID_EMAIL_ADDRESS")[0])
 
     def validate(self, email: str) -> str:
         """
@@ -144,9 +155,12 @@ class MailUtil:
         config variable to pass any relevant arguments to
         email_validator.validate_email() method.
 
-        ValueError is thrown if not valid.
+        EmailValidationException is thrown on invalid email.
         """
 
         validator_args = config_value("EMAIL_VALIDATOR_ARGS") or {}
-        valid = email_validator.validate_email(email, **validator_args)
-        return valid.normalized
+        try:
+            valid = email_validator.validate_email(email, **validator_args)
+            return valid.normalized
+        except ValueError:
+            raise EmailValidateException(get_message("INVALID_EMAIL_ADDRESS")[0])
