@@ -1,12 +1,12 @@
 """
-    flask_security.utils
-    ~~~~~~~~~~~~~~~~~~~~
+flask_security.utils
+~~~~~~~~~~~~~~~~~~~~
 
-    Flask-Security utils module
+Flask-Security utils module
 
-    :copyright: (c) 2012-2019 by Matt Wright.
-    :copyright: (c) 2019-2024 by J. Christopher Wagner (jwag).
-    :license: MIT, see LICENSE for more details.
+:copyright: (c) 2012-2019 by Matt Wright.
+:copyright: (c) 2019-2025 by J. Christopher Wagner (jwag).
+:license: MIT, see LICENSE for more details.
 """
 
 from __future__ import annotations
@@ -25,10 +25,12 @@ import urllib.error
 import warnings
 
 from flask import (
+    Response,
     after_this_request,
     current_app,
     flash,
     g,
+    redirect,
     request,
     render_template,
     session,
@@ -50,9 +52,9 @@ from .proxies import _security, _datastore, _pwd_context, _hashing_context
 from .signals import user_authenticated
 
 if t.TYPE_CHECKING:  # pragma: no cover
-    from flask import Flask, Response
+    from flask import Flask
     from flask.typing import ResponseValue
-    from .datastore import User
+    from flask_security import UserMixin
 
 localize_callback = LocalProxy(lambda: _security.i18n_domain.gettext)
 
@@ -83,7 +85,12 @@ def get_request_attr(name: str) -> t.Any:
     return getattr(g, name, None)
 
 
-def set_request_attr(name, value):
+def set_request_attr(name: str, value: t.Any) -> None:
+    """Set an attribute on Flask's application context global object (g).
+
+    :param name: The key/attribute name to store the value under
+    :param value: The value to store in the application context
+    """
     return setattr(g, name, value)
 
 
@@ -104,31 +111,63 @@ else:
         return response
 
 
-# From a miguel grinberg blog around dealing with 3.12.
-# Our default SQLAlchemy Datetime is naive.
-# Note that most code should call _security.datetime_factory()
-def aware_utcnow():
+def aware_utcnow() -> datetime:
+    """Return a timezone-aware UTC datetime object.
+
+    From a miguel grinberg blog around dealing with 3.12.
+    Our default SQLAlchemy Datetime is naive.
+    Note that most code should call _security.datetime_factory()
+
+    :return: Current UTC datetime with timezone information
+    :rtype: datetime
+    """
     return datetime.now(timezone.utc)
 
 
-def aware_utcfromtimestamp(timestamp):
+def aware_utcfromtimestamp(timestamp: float) -> datetime:
+    """Create timezone-aware UTC datetime from timestamp.
+
+    :param timestamp: Unix timestamp (seconds since epoch)
+    :type timestamp: float
+    :return: UTC datetime with timezone information
+    :rtype: datetime
+    """
     return datetime.fromtimestamp(timestamp, timezone.utc)
 
 
-def naive_utcnow():
+def naive_utcnow() -> datetime:
+    """Return a naive UTC datetime (tzinfo removed).
+
+    :return: Current UTC datetime without timezone information
+    :rtype: datetime
+    """
     return aware_utcnow().replace(tzinfo=None)
 
 
-def naive_utcfromtimestamp(timestamp):
+def naive_utcfromtimestamp(timestamp: float) -> datetime:
+    """Create naive UTC datetime from timestamp.
+
+    :param timestamp: Unix timestamp (seconds since epoch)
+    :type timestamp: float
+    :return: UTC datetime without timezone information
+    :rtype: datetime
+    """
     return aware_utcfromtimestamp(timestamp).replace(tzinfo=None)
 
 
-def find_csrf_field_name():
-    """
-    We need to clear it on logout (since that isn't being done by Flask-WTF).
-    The field name is configurable withing Flask-WTF as well as being
-    overridable.
-    We take the field name from the login_form as set by the configuration.
+def find_csrf_field_name() -> t.Optional[str]:
+    """Retrieve the configured CSRF field name from Flask-WTF form configuration.
+
+    This is needed to properly clear CSRF tokens on logout since Flask-WTF doesn't
+    automatically handle this case. The field name can be configured through
+    Flask-WTF's settings or overridden in form classes.
+
+    :return: Configured CSRF field name if found, None otherwise
+    :rtype: Optional[str]
+
+    Note:
+        Uses the field name from the login form as set by the Flask-WTF configuration.
+        Requires a DummyForm class with Flask-WTF's meta configuration.
     """
     from .forms import DummyForm
 
@@ -138,9 +177,9 @@ def find_csrf_field_name():
     return None
 
 
-def is_user_authenticated(user: User | None) -> bool:
+def is_user_authenticated(user: UserMixin | None) -> bool:
     """
-    return True is user is authenticated.
+    return True if user is authenticated.
 
     With Flask-Login <=0.6.x and Flask-Security <5.4 current_user was always
     set - for non-authenticated users it pointed to an AnonymousUser
@@ -158,7 +197,7 @@ def is_user_authenticated(user: User | None) -> bool:
 
 
 def login_user(
-    user: User,
+    user: UserMixin,
     remember: bool | None = None,
     authn_via: list[str] | None = None,
 ) -> bool:
@@ -188,7 +227,7 @@ def login_user(
 
         old_current_login, new_current_login = (
             user.current_login_at,
-            config_value("DATETIME_FACTORY")(),
+            _security.datetime_factory(),
         )
         old_current_ip, new_current_ip = user.current_login_ip, remote_addr
 
@@ -205,13 +244,13 @@ def login_user(
 
     identity_changed.send(
         current_app._get_current_object(),  # type: ignore[attr-defined]
-        _async_wrapper=current_app.ensure_sync,
+        _async_wrapper=current_app.ensure_sync,  # type: ignore[arg-type]
         identity=Identity(user.fs_uniquifier),
     )
 
     user_authenticated.send(
         current_app._get_current_object(),  # type: ignore[attr-defined]
-        _async_wrapper=current_app.ensure_sync,
+        _async_wrapper=current_app.ensure_sync,  # type: ignore[arg-type]
         user=user,
         authn_via=authn_via,
     )
@@ -267,21 +306,21 @@ def check_and_update_authn_fresh(
                   will set a grace period for which freshness won't be checked.
                   The intent here is that the caller shouldn't get part-way though
                   a set of operations and suddenly be required to authenticate again.
+                  This is not supported for authentication tokens.
     :param method: Optional - if set and == "basic" then will always return True.
                   (since basic-auth sends username/password on every request)
 
     If within.total_seconds() is negative, will always return True (always 'fresh').
     This effectively just disables this entire mechanism.
 
+    within.total_seconds() == 0 results in undefined behavior.
+
     If "fs_gexp" is in the session and the current timestamp is less than that,
     return True and extend grace time (i.e. set fs_gexp to current time + grace).
 
-    If not within the grace period, and within.total_seconds() is 0,
-    return False (not fresh).
-
-    Be aware that for this to work, sessions and therefore session cookies
-    must be functioning and being sent as part of the request. If the required
-    state isn't in the session cookie then return False (not 'fresh').
+    Be aware that for this to work, state is required to be sent from the client.
+    Flask security adds this state to the session (cookie) and the auth token.
+    Without this state, 'False' is always returned - (not fresh).
 
     .. warning::
         Be sure the caller is already authenticated PRIOR to calling this method.
@@ -290,6 +329,10 @@ def check_and_update_authn_fresh(
 
     .. versionchanged:: 4.0.0
         Added `method` parameter.
+
+    .. versionchanged:: 5.5.0
+        Grab 'Primary Authenticated At' from  request_attrs
+        which is set from either session or auth token
     """
 
     if method == "basic":
@@ -299,8 +342,8 @@ def check_and_update_authn_fresh(
         # this means 'always fresh'
         return True
 
-    if "fs_paa" not in session:
-        # No session, you can't play.
+    if not (paa := get_request_attr("fs_paa")):
+        # No recorded primary authenticated at time, you can't play.
         return False
 
     now = naive_utcnow()
@@ -313,12 +356,7 @@ def check_and_update_authn_fresh(
             session["fs_gexp"] = grace_ts
             return True
 
-    # Special case 0 - return False always, but set grace period.
-    if within.total_seconds() == 0:
-        session["fs_gexp"] = grace_ts
-        return False
-
-    authn_time = naive_utcfromtimestamp(session["fs_paa"])
+    authn_time = naive_utcfromtimestamp(paa)
     # allow for some time drift where it's possible authn_time is in the future
     # but let's be cautious and not allow arbitrary future times
     delta = now - authn_time
@@ -361,7 +399,7 @@ def verify_password(password: str | bytes, password_hash: str | bytes) -> bool:
     return _pwd_context.verify(password, password_hash)
 
 
-def verify_and_update_password(password: str | bytes, user: User) -> bool:
+def verify_and_update_password(password: str | bytes, user: UserMixin) -> bool:
     """Returns ``True`` if the password is valid for the specified user.
 
     Additionally, the hashed password in the database is updated if the
@@ -380,14 +418,21 @@ def verify_and_update_password(password: str | bytes, user: User) -> bool:
         :meth:`.UserMixin.verify_and_update_password`
 
     """
+    # Capture the original input in case we need to pass the unaltered
+    # value to hash_password if the hashing algorithm has changed
+    input_password = password
+
     if use_double_hash(user.password):
-        verified = _pwd_context.verify(get_hmac(password), user.password)
+        password = get_hmac(password)
+        if _pwd_context.identify(user.password) == "bcrypt":
+            password = password[:72]
+        verified = _pwd_context.verify(password, user.password)
     else:
         # Try with original password.
         verified = _pwd_context.verify(password, user.password)
 
-    if verified and _pwd_context.needs_update(user.password):
-        user.password = hash_password(password)
+    if verified and (user.password is None or _pwd_context.needs_update(user.password)):
+        user.password = hash_password(input_password)
         _datastore.put(user)
     return verified
 
@@ -406,6 +451,9 @@ def hash_password(password: str | bytes) -> str:
 
     .. versionadded:: 2.0.2
 
+    .. versionchanged:: 5.7.0
+       Explicit check for bcrypt truncation
+
     :param password: The plaintext password to hash
     """
     if use_double_hash():
@@ -413,6 +461,11 @@ def hash_password(password: str | bytes) -> str:
 
     # Passing in options as part of hash is deprecated in passlib 1.7
     # and new algorithms like argon2 don't even support it.
+    if config_value("PASSWORD_HASH") == "bcrypt":
+        # bcrypt - OWASP says truncation concerns are negligible:
+        # https://cheatsheetseries.owasp.org/cheatsheets/Password_Storage_Cheat_Sheet.html#input-limits-of-bcrypt
+        password = password[:72]
+
     return _pwd_context.hash(
         password,
         **config_value("PASSWORD_HASH_OPTIONS", default={}).get(
@@ -421,21 +474,37 @@ def hash_password(password: str | bytes) -> str:
     )
 
 
-def encode_string(string):
+def encode_string(string: t.Union[str, bytes]) -> bytes:
     """Encodes a string to bytes, if it isn't already.
 
-    :param string: The string to encode"""
-
+    :param string: The string to encode
+    :return: UTF-8 encoded bytes
+    """
     if isinstance(string, str):
         string = string.encode("utf-8")
     return string
 
 
-def hash_data(data):
+def hash_data(data: t.Union[str, bytes]) -> str:
+    """Hashes input data after ensuring proper encoding.
+
+    :param data: Input data to hash (will be encoded if not already bytes)
+    :return: Hashed data as bytes
+
+    Note: Uses application's configured _hashing_context
+    """
     return _hashing_context.hash(encode_string(data))
 
 
-def verify_hash(hashed_data, compare_data):
+def verify_hash(hashed_data: bytes, compare_data: t.Union[str, bytes]) -> bool:
+    """Verifies data against a previously hashed value.
+
+    :param hashed_data: Previously hashed data to compare against
+    :param compare_data: Input data to verify (will be encoded if not already bytes)
+    :return: True if data matches hash, False otherwise
+
+    Note: Uses application's configured _hashing_context
+    """
     return _hashing_context.verify(encode_string(compare_data), hashed_data)
 
 
@@ -446,9 +515,6 @@ def suppress_form_csrf():
     If app doesn't want CSRF for unauth endpoints then check if caller is authenticated
     or not (many endpoints can be called either way).
     """
-    if get_request_attr("fs_ignore_csrf"):
-        # This is the case where CsrfProtect was already called (e.g. @auth_required)
-        return {"csrf": False}
     if config_value("CSRF_IGNORE_UNAUTH_ENDPOINTS") and not is_user_authenticated(
         current_user
     ):
@@ -485,7 +551,7 @@ def parse_auth_token(auth_token: str) -> dict[str, t.Any]:
     # Version 5 and up are already a dict (with a version #)
     if isinstance(raw_data, dict):
         # new format - starting at ver=5
-        if not all(k in raw_data for k in ["ver", "uid", "exp", "sid"]):
+        if not all(k in raw_data for k in ["ver", "uid", "exp"]):
             raise ValueError("Token missing keys")
         tdata = raw_data
         if ts := tdata.get("exp"):
@@ -536,10 +602,25 @@ def get_url(endpoint_or_url: str, qparams: dict[str, str] | None = None) -> str:
         return url
 
 
-def slash_url_suffix(url, suffix):
-    """Adds a slash either to the beginning or the end of a suffix
-    (which is to be appended to a URL), depending on whether or not
-    the URL ends with a slash."""
+def slash_url_suffix(url: str, suffix: str) -> str:
+    """
+    Formats a suffix to be appended to a URL, ensuring proper slash placement.
+
+    If the given `url` ends with a slash, this function adds a trailing slash
+    to the `suffix`.
+    Otherwise, it adds a leading slash to the `suffix`.
+    This helps prevent double slashes or missing slashes when constructing URLs.
+
+    :param url: The base URL to which the suffix will be appended.
+    :param suffix: The suffix to be appended to the URL.
+    :return: The formatted suffix with the appropriate leading or trailing slash.
+
+    Example:
+        >>> slash_url_suffix("https://example.com/api", "v1")
+        '/v1'
+        >>> slash_url_suffix("https://example.com/api/", "v1")
+        'v1/'
+    """
     return url.endswith("/") and f"{suffix}/" or f"/{suffix}"
 
 
@@ -563,7 +644,18 @@ def transform_url(
     return urlunsplit(link_parse._replace(**kwargs))
 
 
-def get_security_endpoint_name(endpoint):
+def get_security_endpoint_name(endpoint: str) -> str:
+    """
+    Returns the fully qualified endpoint name by combining the blueprint name
+    and the endpoint.
+
+    :param endpoint: The endpoint name to be combined with the blueprint name.
+    :return: The fully qualified endpoint name in the format '<blueprint>.<endpoint>'.
+
+    Example:
+        >>> get_security_endpoint_name("login")
+        'my_blueprint.login'
+    """
     return f"{config_value('BLUEPRINT_NAME')}.{endpoint}"
 
 
@@ -575,8 +667,8 @@ def url_for_security(endpoint: str, **values: t.Any) -> str:
     :param _external: if set to `True`, an absolute URL is generated. Server
       address can be changed via `SERVER_NAME` configuration variable which
       defaults to `localhost`.
-    :param _anchor: if provided this is added as anchor to the URL.
-    :param _method: if provided this explicitly specifies an HTTP method.
+    :param _anchor: if provided, this is added as anchor to the URL.
+    :param _method: if provided, this explicitly specifies an HTTP method.
     """
     endpoint = get_security_endpoint_name(endpoint)
     # mypy is complaining about this - but I think it's wrong?
@@ -584,10 +676,51 @@ def url_for_security(endpoint: str, **values: t.Any) -> str:
 
 
 def validate_redirect_url(url: str) -> bool:
-    """Validate that the URL for redirect is relative.
+    """Validate redirect URL
+    In the default configuration only redirects to the same domain (and scheme)
+    are allowed.
+    The REDIRECT_ALLOW_SUBDOMAINS allows ANY subdomain of SERVER_NAME
+     to be a redirect target.
+    The REDIRECT_BASE_DOMAIN and REDIRECT_ALLOWED_SUBDOMAINS allow specifying 'side'
+    redirects.
+    """
+
+    if url is None or url.strip() == "":
+        return False
+    url_next = urlsplit(url)
+    url_base = urlsplit(request.host_url)
+    if (url_next.netloc or url_next.scheme) and url_next.netloc != url_base.netloc:
+        base_domain = current_app.config.get("SERVER_NAME")
+        if (
+            config_value("REDIRECT_ALLOW_SUBDOMAINS")
+            and base_domain
+            and (
+                url_next.netloc == base_domain
+                or url_next.netloc.endswith(f".{base_domain}")
+            )
+        ):
+            return True
+        base_domain = config_value("REDIRECT_BASE_DOMAIN")
+        if base_domain:
+            allowable = [
+                f"{sub}.{base_domain}"
+                for sub in config_value("REDIRECT_ALLOWED_SUBDOMAINS")
+            ]
+            return url_next.netloc in allowable
+        return False
+    return True
+
+
+def get_post_action_redirect(
+    config_key: str, next_loc: FlaskForm | MultiDict | dict | None
+) -> str:
+    """
+    There is a security angle here - the result of this method is
+    sent to Flask::redirect() - and we need to be sure that it can't be
+    interpreted as a user-input external URL - that would mean we would
+    have an 'open-redirect' vulnerability.
+
     Allowing an absolute redirect is a security issue - a so-called open-redirect.
-    Note that by default Werkzeug will always take this URL and make it relative
-    when setting the Location header - but that behavior can be overridden.
 
     The complexity here is that urlsplit() does pretty well, but browsers even today
     May 2021 are very lenient in what they accept as URLs - for example:
@@ -611,35 +744,8 @@ def validate_redirect_url(url: str) -> bool:
     that all the above URLs are valid.
     By the time we get the URL, it has been unencoded - so we can't really determine
     if it is 'valid' since it appears that '/'s can appear in the URL if escaped.
-    """
-    if url is None or url.strip() == "":
-        return False
-    url_next = urlsplit(url)
-    url_base = urlsplit(request.host_url)
-    if (url_next.netloc or url_next.scheme) and url_next.netloc != url_base.netloc:
-        base_domain = current_app.config.get("SERVER_NAME")
-        if (
-            config_value("REDIRECT_ALLOW_SUBDOMAINS")
-            and base_domain
-            and (
-                url_next.netloc == base_domain
-                or url_next.netloc.endswith(f".{base_domain}")
-            )
-        ):
-            return True
-        else:
-            return False
-    return True
 
-
-def get_post_action_redirect(
-    config_key: str, next_loc: FlaskForm | MultiDict | dict | None
-) -> str:
-    """
-    There is a security angle here - the result of this method is
-    sent to Flask::redirect() - and we need to be sure that it can't be
-    interpreted as a user-input external URL - that would mean we would
-    have an 'open-redirect' vulnerability.
+    The solution is to simply 'quote' the path.
     """
     rurl = propagate_next(find_redirect(config_key), next_loc)
     (scheme, netloc, path, query, fragment) = urlsplit(rurl)
@@ -799,7 +905,7 @@ def send_mail(subject, recipient, template, **context):
     if isinstance(sender, LocalProxy):
         sender = sender._get_current_object()
 
-    _security._mail_util.send_mail(
+    _security.mail_util.send_mail(
         template,
         subject,
         recipient,
@@ -893,7 +999,7 @@ def get_identity_attributes(app: Flask | None = None) -> list[str]:
 
 
 def get_identity_attribute(attr: str, app: Flask | None = None) -> dict[str, t.Any]:
-    """Given an user_identity_attribute, return the defining dict.
+    """Given a user_identity_attribute, return the defining dict.
     A bit annoying since USER_IDENTITY_ATTRIBUTES is a list of dict
     where each dict has just one key.
     """
@@ -912,7 +1018,7 @@ def lookup_identity(identity):
     """
     Lookup identity in DB.
     This loops through, in order, :py:data:`SECURITY_USER_IDENTITY_ATTRIBUTES`,
-    and first calls the mapper function to validate/normalize.
+    and first calls the mapper function to normalize.
     Then the db.find_user is called on the specified user model attribute.
     """
     for mapping in config_value("USER_IDENTITY_ATTRIBUTES"):
@@ -928,19 +1034,21 @@ def lookup_identity(identity):
 
 
 def uia_phone_mapper(identity: str) -> str | None:
-    """Used to match identity as a phone number. This is a simple proxy
-    to :py:class:`PhoneUtil`
+    """Used to normalize a phone number. This is a simple proxy
+    to :py:meth:`PhoneUtil.get_canonical_form`
 
     See :py:data:`SECURITY_USER_IDENTITY_ATTRIBUTES`.
 
     .. versionadded:: 3.4.0
     """
-    ph = _security._phone_util.get_canonical_form(identity)
+    ph = _security.phone_util.get_canonical_form(identity)
     return ph
 
 
 def uia_email_mapper(identity: str) -> str | None:
-    """Used to match identity as an email.
+    """Used to normalize identity as an email.
+    This is a simple proxy
+    to :py:meth:`MailUtil.normalize`
 
     :return: Normalized email or None if not valid email.
 
@@ -950,20 +1058,20 @@ def uia_email_mapper(identity: str) -> str | None:
     """
 
     try:
-        return _security._mail_util.normalize(identity)
+        return _security.mail_util.normalize(identity)
     except ValueError:
         return None
 
 
 def uia_username_mapper(identity: str) -> str | None:
-    """Used to match identity as a username. This is a simple proxy
-    to :py:class:`UsernameUtil`
+    """Used to normalize a username. This is a simple proxy
+    to :py:meth:`UsernameUtil.normalize`
 
     See :py:data:`SECURITY_USER_IDENTITY_ATTRIBUTES`.
 
     .. versionadded:: 4.1.0
     """
-    return _security._username_util.normalize(identity)
+    return _security.username_util.normalize(identity)
 
 
 def use_double_hash(password_hash=None):
@@ -1037,7 +1145,7 @@ def csrf_cookie_handler(response: Response) -> Response:
     elif current_app.config["WTF_CSRF_TIME_LIMIT"]:
         current_cookie = request.cookies.get(csrf_cookie_name, None)
         if current_cookie:
-            # Lets make sure it isn't expired if app doesn't set TIME_LIMIT to None.
+            # Let's make sure it isn't expired if app doesn't set TIME_LIMIT to None.
             try:
                 csrf.validate_csrf(current_cookie)
             except ValidationError:
@@ -1171,7 +1279,7 @@ def json_error_response(
     return response_json
 
 
-def default_render_template(*args, **kwargs):
+def default_render_template(*args: t.Any, **kwargs: t.Any) -> str:
     return render_template(*args, **kwargs)
 
 
@@ -1188,9 +1296,11 @@ class SmsSenderBaseClass(metaclass=abc.ABCMeta):
 
 
 class DummySmsSender(SmsSenderBaseClass):
-    def send_sms(self, from_number, to_number, msg):  # pragma: no cover
+    def send_sms(
+        self, from_number: str, to_number: str, msg: str
+    ) -> None:  # pragma: no cover
         """Do nothing."""
-        return
+        return None
 
 
 class SmsSenderFactory:
@@ -1216,7 +1326,7 @@ try:  # pragma: no cover
             self.account_sid = config_value("SMS_SERVICE_CONFIG")["ACCOUNT_SID"]
             self.auth_token = config_value("SMS_SERVICE_CONFIG")["AUTH_TOKEN"]
 
-        def send_sms(self, from_number, to_number, msg):
+        def send_sms(self, from_number: str, to_number: str, msg: str) -> None:
             """Send message via twilio account."""
             client = Client(self.account_sid, self.auth_token)
             client.messages.create(to=to_number, from_=from_number, body=msg)
@@ -1345,3 +1455,31 @@ def pwned(password: str) -> int:
 
     entries = dict(map(convert_password_tuple, raw.upper().split("\r\n")))
     return entries.get(sha1[5:].upper(), 0)
+
+
+def handle_already_auth(form, payload=None):
+    """
+    Allow already authenticated users. For GET this is useful for
+    single-page-applications on refresh - session still active but need to
+    access user info and csrf-token.
+    For GET with forms - redirect to POST_LOGIN_VIEW.
+    For POST - redirects to POST_LOGIN_VIEW (forms) or returns 400 (json).
+
+    This does NOT use get_post_login_redirect() so that it doesn't look at
+    'next' - which can cause infinite redirect loops
+    (see test_basic::test_authenticated_loop)
+
+    While it's tempting to try to logout the current user and login the
+    new requested user - that simply doesn't work with CSRF.
+    """
+    if _security._want_json(request):
+        if request.method == "POST":
+            payload = json_error_response(
+                errors=get_message("ANONYMOUS_USER_REQUIRED")[0]
+            )
+            return _security._render_json(payload, 400, None, None)
+        else:
+            form.user = current_user
+            return base_render_json(form, additional=payload)
+    else:
+        return redirect(get_url(config_value("POST_LOGIN_VIEW")))
