@@ -4,7 +4,7 @@ test_change_email
 
 Change email functionality tests
 
-:copyright: (c) 2024-2024 by J. Christopher Wagner (jwag).
+:copyright: (c) 2024-2025 by J. Christopher Wagner (jwag).
 :license: MIT, see LICENSE for more details.
 """
 
@@ -47,7 +47,7 @@ def capture_change_email_requests():
 
 
 @pytest.mark.settings(change_email_error_view="/change-email")
-def test_ce(app, clients, get_message):
+def test_ce(app, clients, get_message, outbox):
     client = clients
 
     @change_email_confirmed.connect_via(app)
@@ -60,15 +60,15 @@ def test_ce(app, clients, get_message):
     with capture_change_email_requests() as ce_requests:
         response = client.post("/change-email", data={"email": "<EMAIL>"})
         assert get_message("INVALID_EMAIL_ADDRESS") in response.data
-        assert not app.mail.outbox
+        assert len(outbox) == 0
 
         response = client.post("/change-email", data=dict(email="matt2@lp.com"))
         msg = get_message("CHANGE_EMAIL_SENT", email="matt2@lp.com")
         assert msg in response.data
         assert "matt2@lp.com" == ce_requests[0]["new_email"]
         token = ce_requests[0]["token"]
-    assert len(app.mail.outbox) == 1
-    assert app.config["SECURITY_CHANGE_EMAIL_WITHIN"] in app.mail.outbox[0].body
+    assert len(outbox) == 1
+    assert app.config["SECURITY_CHANGE_EMAIL_WITHIN"] in outbox[0].body
 
     response = client.get("/change-email/" + token, follow_redirects=True)
     assert get_message("CHANGE_EMAIL_CONFIRMED") in response.data
@@ -84,7 +84,7 @@ def test_ce(app, clients, get_message):
     assert flashes[0]["message"].encode("utf-8") == get_message("API_ERROR")
 
 
-def test_ce_json(app, client, get_message):
+def test_ce_json(app, client, get_message, outbox):
     headers = {"Accept": "application/json", "Content-Type": "application/json"}
 
     @change_email_confirmed.connect_via(app)
@@ -99,14 +99,14 @@ def test_ce_json(app, client, get_message):
         assert response.json["response"]["errors"][0].encode("utf=8") == get_message(
             "INVALID_EMAIL_ADDRESS"
         )
-        assert not app.mail.outbox
+        assert len(outbox) == 0
 
         response = client.post("/change-email", json=dict(email="matt2@lp.com"))
         assert response.status_code == 200
         assert response.json["response"]["current_email"] == "matt@lp.com"
         assert "matt2@lp.com" == ce_requests[0]["new_email"]
         token = ce_requests[0]["token"]
-    assert len(app.mail.outbox) == 1
+    assert len(outbox) == 1
 
     response = client.get(
         "/change-email/" + token, headers=headers, follow_redirects=True
@@ -137,7 +137,7 @@ def test_expired_token(client, get_message):
     assert msg in response.data
 
 
-def test_template(app, client, get_message):
+def test_template(app, client, get_message, outbox):
     # Check contents of email template - this uses a test template
     # in order to check all context vars since the default template
     # doesn't have all of them.
@@ -146,8 +146,7 @@ def test_template(app, client, get_message):
     with capture_change_email_requests() as ce_requests:
         client.post("/change-email", data=dict(email="matt2@lp.com"))
         # check email
-        outbox = app.mail.outbox
-        assert outbox[0].to[0] == "matt2@lp.com"
+        assert outbox[0].recipients[0] == "matt2@lp.com"
         matcher = re.findall(r"\w+:.*", outbox[0].body, re.IGNORECASE)
         # should be 4 - link, email, token, config item
         assert matcher[1].split(":")[1] == "matt@lp.com"
@@ -162,7 +161,7 @@ def test_template(app, client, get_message):
 
 
 @pytest.mark.settings(return_generic_responses=True)
-def test_generic_response(app, client, get_message):
+def test_generic_response(app, client, get_message, outbox):
     authenticate(client, email="matt@lp.com")
     with capture_change_email_requests():
         # first try bad formatted email - should get detailed error
@@ -176,7 +175,7 @@ def test_generic_response(app, client, get_message):
         msg = get_message("CHANGE_EMAIL_SENT", email="gal@lp.com")
         assert msg in response.data
         # but no email was actually sent
-        assert not app.mail.outbox
+        assert len(outbox) == 0
 
 
 @pytest.mark.settings(
